@@ -539,8 +539,6 @@ meta_screen_new (MetaDisplay *display,
   screen->vertical_workspaces = FALSE;
   screen->starting_corner = META_SCREEN_TOPLEFT;
 
-  screen->showing_desktop = FALSE;
-
   screen->compositor_windows = NULL;
   screen->damage_region = None;
   screen->root_picture = None;
@@ -548,15 +546,31 @@ meta_screen_new (MetaDisplay *display,
   screen->trans_picture = None;
   
   {
+    XFontStruct *font_info;
     XGCValues gc_values;
+    gulong value_mask = 0;
     
     gc_values.subwindow_mode = IncludeInferiors;
+    value_mask |= GCSubwindowMode;
     gc_values.function = GXinvert;
+    value_mask |= GCFunction;
     gc_values.line_width = META_WIREFRAME_XOR_LINE_WIDTH;
-    
+    value_mask |= GCLineWidth;
+
+    font_info = XLoadQueryFont (screen->display->xdisplay, "fixed");
+
+    if (font_info != NULL)
+      {
+        gc_values.font = font_info->fid;
+        value_mask |= GCFont;
+        XFreeFontInfo (NULL, font_info, 1);
+      }
+    else
+      meta_warning ("xserver doesn't have 'fixed' font.\n");
+
     screen->root_xor_gc = XCreateGC (screen->display->xdisplay,
                                      screen->xroot,
-                                     GCSubwindowMode | GCFunction | GCLineWidth,
+                                     value_mask,
                                      &gc_values);
   }
   
@@ -604,7 +618,7 @@ meta_screen_new (MetaDisplay *display,
   /* Screens must have at least one workspace at all times,
    * so create that required workspace.
    */
-  meta_workspace_activate (meta_workspace_new (screen));
+  meta_workspace_activate (meta_workspace_new (screen), timestamp);
   update_num_workspaces (screen);
   
   set_workspace_names (screen);
@@ -641,7 +655,7 @@ meta_screen_new (MetaDisplay *display,
                                                 current_workspace);
     
     if (space != NULL)
-      meta_workspace_activate (space);
+      meta_workspace_activate (space, timestamp);
   }
 
   meta_compositor_manage_screen (screen->display->compositor,
@@ -657,6 +671,7 @@ void
 meta_screen_free (MetaScreen *screen)
 {
   MetaDisplay *display;
+  XGCValues gc_values = { 0 };
 
   display = screen->display;
 
@@ -707,9 +722,22 @@ meta_screen_free (MetaScreen *screen)
   if (screen->work_area_idle != 0)
     g_source_remove (screen->work_area_idle);
 
+
+  if (XGetGCValues (screen->display->xdisplay,
+                    screen->root_xor_gc,
+                    GCFont,
+                    &gc_values))
+    {
+      XUnloadFont (screen->display->xdisplay,
+                   gc_values.font);
+    }
+
   XFreeGC (screen->display->xdisplay,
            screen->root_xor_gc);
   
+  if (screen->xinerama_infos)
+    g_free (screen->xinerama_infos);
+  
   g_free (screen->screen_name);
   g_free (screen);
 
@@ -1077,7 +1105,7 @@ update_num_workspaces (MetaScreen *screen)
     }
 
   if (need_change_space)
-    meta_workspace_activate (last_remaining);
+    meta_workspace_activate (last_remaining, meta_display_get_current_time_roundtrip (screen->display));
 
   /* Should now be safe to free the workspaces */
   tmp = extras;
@@ -1165,6 +1193,8 @@ meta_screen_ensure_tab_popup (MetaScreen *screen,
       entries[i].title = window->title;
       entries[i].icon = window->icon;
       entries[i].blank = FALSE;
+      entries[i].minimized = !meta_window_showing_on_its_workspace (window);
+      entries[i].demands_attention = window->wm_state_demands_attention;
       
       if (!window->minimized || !meta_window_get_icon_geometry (window, &r))
         meta_window_get_outer_rect (window, &r);
@@ -1267,6 +1297,8 @@ meta_screen_ensure_workspace_popup (MetaScreen *screen)
           entries[i].icon = NULL;
           entries[i].blank = TRUE;
         }
+      entries[i].minimized = FALSE;
+      entries[i].demands_attention = FALSE;
 
       ++i;
     }
@@ -1769,7 +1801,7 @@ meta_create_offscreen_window (Display *xdisplay,
                         0,
                         CopyFromParent,
                         CopyFromParent,
-                        CopyFromParent,
+                        (Visual *)CopyFromParent,
                         CWOverrideRedirect | CWEventMask,
                         &attrs);
 }
@@ -2166,12 +2198,12 @@ meta_screen_resize (MetaScreen *screen,
   meta_screen_foreach_window (screen, meta_screen_resize_func, 0);
 }
 
-static void
-update_showing_desktop_hint (MetaScreen *screen)
+void
+meta_screen_update_showing_desktop_hint (MetaScreen *screen)
 {
   unsigned long data[1];
 
-  data[0] = screen->showing_desktop ? 1 : 0;  
+  data[0] = screen->active_workspace->showing_desktop ? 1 : 0;
       
   meta_error_trap_push (screen->display);
   XChangeProperty (screen->display->xdisplay, screen->xroot,
@@ -2187,6 +2219,10 @@ queue_windows_showing (MetaScreen *screen)
   GSList *windows;
   GSList *tmp;
 
+  /* Must operate on all windows on display instead of just on the
+   * active_workspace's window list, because the active_workspace's
+   * window list may not contain the on_all_workspace windows.
+   */
   windows = meta_display_list_windows (screen->display);
 
   tmp = windows;
@@ -2204,13 +2240,13 @@ queue_windows_showing (MetaScreen *screen)
 }
 
 void
-meta_screen_minimize_all_except (MetaScreen *screen,
-                                 MetaWindow *keep)
+meta_screen_minimize_all_on_active_workspace_except (MetaScreen *screen,
+                                                     MetaWindow *keep)
 {
-  GSList *windows;
-  GSList *tmp;
-  
-  windows = meta_display_list_windows (screen->display);
+  GList *windows;
+  GList *tmp;
+
+  windows = screen->active_workspace->windows;
   
   tmp = windows;
   while (tmp != NULL)
@@ -2224,38 +2260,57 @@ meta_screen_minimize_all_except (MetaScreen *screen,
       
       tmp = tmp->next;
     }
-  
-  g_slist_free (windows);
 }
 
 void
-meta_screen_show_desktop (MetaScreen *screen)
+meta_screen_show_desktop (MetaScreen *screen, 
+                          Time        timestamp)
 {
-  if (screen->showing_desktop)
+  GList *windows;
+
+  if (screen->active_workspace->showing_desktop)
     return;
-
-  screen->showing_desktop = TRUE;
-
+  
+  screen->active_workspace->showing_desktop = TRUE;
+  
   queue_windows_showing (screen);
 
-  update_showing_desktop_hint (screen);
+  /* Focus the most recently used META_WINDOW_DESKTOP window, if there is one;
+   * see bug 159257.
+   */
+  windows = screen->active_workspace->mru_list;
+  while (windows != NULL)
+    {
+      MetaWindow *w = windows->data;
+      
+      if (w->screen == screen  && 
+          w->type == META_WINDOW_DESKTOP)
+        {
+          meta_window_focus (w, timestamp);
+          break;
+        }
+      
+      windows = windows->next;
+    }
 
-  meta_workspace_focus_top_window (screen->active_workspace, NULL);
+  
+  meta_screen_update_showing_desktop_hint (screen);
 }
 
 void
 meta_screen_unshow_desktop (MetaScreen *screen)
 {
-  if (!screen->showing_desktop)
+  if (!screen->active_workspace->showing_desktop)
     return;
 
-  screen->showing_desktop = FALSE;
-  
+  screen->active_workspace->showing_desktop = FALSE;
+
   queue_windows_showing (screen);
 
-  update_showing_desktop_hint (screen);
+  meta_screen_update_showing_desktop_hint (screen);
 }
 
+
 #ifdef HAVE_STARTUP_NOTIFICATION
 static gboolean startup_sequence_timeout (void *data);
 
